(*==========================================================================
 *
 *  Copyright (C) 1995-1997 Microsoft Corporation.  All Rights Reserved.
 *
 *  File:       dsetup.h
 *  Content:    DirectXSetup, error codes and flags
 ***************************************************************************)
unit dsetup;

{$MINENUMSIZE 4}
{$ALIGN ON}

interface

uses
  windows;

type
  PDLSVERSION = ^TDLSVERSION;
  TDLSVERSION = packed record
    dwVersionMS: DWORD;
    dwVersionLS: DWORD;
  end;
  DLSVERSION = TDLSVERSION;

const
  FOURCC_VERS: array[ 0..3 ] of Char = ('v','e','r','s');

  // DSETUP Error Codes, must remain compatible with previous setup.
  DSETUPERR_SUCCESS_RESTART     = HRESULT(1);
  DSETUPERR_SUCCESS             = HRESULT(0);
  DSETUPERR_BADWINDOWSVERSION   = HRESULT(-1);
  DSETUPERR_SOURCEFILENOTFOUND  = HRESULT(-2);
  DSETUPERR_BADSOURCESIZE       = HRESULT(-3);
  DSETUPERR_BADSOURCETIME       = HRESULT(-4);
  DSETUPERR_NOCOPY              = HRESULT(-5);
  DSETUPERR_OUTOFDISKSPACE      = HRESULT(-6);
  DSETUPERR_CANTFINDINF         = HRESULT(-7);
  DSETUPERR_CANTFINDDIR         = HRESULT(-8);
  DSETUPERR_INTERNAL            = HRESULT(-9);
  DSETUPERR_NTWITHNO3D          = HRESULT(-10);  // REM: obsolete, you'll never see this
  DSETUPERR_UNKNOWNOS           = HRESULT(-11);
  DSETUPERR_USERHITCANCEL       = HRESULT(-12);
  DSETUPERR_NOTPREINSTALLEDONNT = HRESULT(-13);
  DSETUPERR_NEWERVERSION	= HRESULT(-14);

  // DSETUP flags. DirectX 5.0 apps should use these flags only.
  DSETUP_DDRAWDRV         = $00000008;      // install DirectDraw Drivers
  DSETUP_DSOUNDDRV        = $00000010;      // install DirectSound Drivers
  DSETUP_DXCORE           = $00010000;      // install DirectX runtime
  DSETUP_DIRECTX = DSETUP_DXCORE or DSETUP_DDRAWDRV or DSETUP_DSOUNDDRV;
  DSETUP_TESTINSTALL      = $00020000;      // just test install, don't do anything
  DSETUP_USEROLDERFLAG    = $02000000;      // enable return DSETUPERR_NEWERVERSION
  // Bug #22730
     //  DSETUP_NTINSTALL  $00080000  // install on Win2K platform

  // These OBSOLETE flags are here for compatibility with pre-DX5 apps only.
  // They are present to allow DX3 apps to be recompiled with DX5 and still work.
  // DO NOT USE THEM for DX5. They will go away in future DX releases.
  DSETUP_DDRAW            = $00000001;      // OBSOLETE. install DirectDraw
  DSETUP_DSOUND           = $00000002;      // OBSOLETE. install DirectSound
  DSETUP_DPLAY            = $00000004;      // OBSOLETE. install DirectPlay
  DSETUP_DPLAYSP          = $00000020;      // OBSOLETE. install DirectPlay Providers
  DSETUP_DVIDEO           = $00000040;      // OBSOLETE. install DirectVideo
  DSETUP_D3D              = $00000200;      // OBSOLETE. install Direct3D
  DSETUP_DINPUT           = $00000800;      // OBSOLETE. install DirectInput
  DSETUP_DIRECTXSETUP     = $00001000;      // OBSOLETE. install DirectXSetup DLL's
  DSETUP_NOUI             = $00002000;      // OBSOLETE. install DirectX with NO UI
  DSETUP_PROMPTFORDRIVERS = $10000000;      // OBSOLETE. prompt when replacing display/audio drivers
  DSETUP_RESTOREDRIVERS   = $20000000;      // OBSOLETE. restore display/audio drivers

  //******************************************************************
  // DirectX Setup Callback mechanism
  //******************************************************************

  // DSETUP Message Info Codes, passed to callback as Reason parameter.
  DSETUP_CB_MSG_NOMESSAGE                 = 0;
  DSETUP_CB_MSG_CANTINSTALL_UNKNOWNOS     = 1;
  DSETUP_CB_MSG_CANTINSTALL_NT            = 2;
  DSETUP_CB_MSG_CANTINSTALL_BETA          = 3;
  DSETUP_CB_MSG_CANTINSTALL_NOTWIN32      = 4;
  DSETUP_CB_MSG_CANTINSTALL_WRONGLANGUAGE = 5;
  DSETUP_CB_MSG_CANTINSTALL_WRONGPLATFORM = 6;
  DSETUP_CB_MSG_PREINSTALL_NT             = 7;
  DSETUP_CB_MSG_NOTPREINSTALLEDONNT       = 8;
  DSETUP_CB_MSG_SETUP_INIT_FAILED         = 9;
  DSETUP_CB_MSG_INTERNAL_ERROR            = 10;
  DSETUP_CB_MSG_CHECK_DRIVER_UPGRADE      = 11;
  DSETUP_CB_MSG_OUTOFDISKSPACE            = 12;
  DSETUP_CB_MSG_BEGIN_INSTALL             = 13;
  DSETUP_CB_MSG_BEGIN_INSTALL_RUNTIME     = 14;
  DSETUP_CB_MSG_BEGIN_INSTALL_DRIVERS     = 15;
  DSETUP_CB_MSG_BEGIN_RESTORE_DRIVERS     = 16;
  DSETUP_CB_MSG_FILECOPYERROR             = 17;


  DSETUP_CB_UPGRADE_TYPE_MASK      = $000F;
  DSETUP_CB_UPGRADE_KEEP           = $0001;
  DSETUP_CB_UPGRADE_SAFE           = $0002;
  DSETUP_CB_UPGRADE_FORCE          = $0004;
  DSETUP_CB_UPGRADE_UNKNOWN        = $0008;

  DSETUP_CB_UPGRADE_HASWARNINGS    = $0100;
  DSETUP_CB_UPGRADE_CANTBACKUP     = $0200;

  DSETUP_CB_UPGRADE_DEVICE_ACTIVE  = $0800;

  DSETUP_CB_UPGRADE_DEVICE_DISPLAY = $1000;
  DSETUP_CB_UPGRADE_DEVICE_MEDIA   = $2000;

type
  PDSETUP_CB_UPGRADEINFO = ^TDSETUP_CB_UPGRADEINFO;
  TDSETUP_CB_UPGRADEINFO = record
    UpgradeFlags: DWORD;
  end;
  DSETUP_CB_UPGRADEINFO = TDSETUP_CB_UPGRADEINFO;


  PDSETUP_CB_FILECOPYERROR = ^TDSETUP_CB_FILECOPYERROR;
  TDSETUP_CB_FILECOPYERROR = record
    dwError: DWORD;
  end;
  DSETUP_CB_FILECOPYERROR = TDSETUP_CB_FILECOPYERROR;

  //
  // Data Structures
  //
  PDIRECTXREGISTERAPPA = ^TDIRECTXREGISTERAPPA;
  TDIRECTXREGISTERAPPA = record
    dwSize:               DWORD;
    dwFlags:              DWORD;
    lpszApplicationName:  LPSTR;
    lpGUID:               PGUID;
    lpszFilename:         LPSTR;
    lpszCommandLine:      LPSTR;
    lpszPath:             LPSTR;
    lpszCurrentDirectory: LPSTR;
  end;
  DIRECTXREGISTERAPPA = TDIRECTXREGISTERAPPA;

  PDIRECTXREGISTERAPP2A = ^TDIRECTXREGISTERAPP2A;
  TDIRECTXREGISTERAPP2A = record
    dwSize:               DWORD;
    dwFlags:              DWORD;
    lpszApplicationName:  LPSTR;
    lpGUID:               PGUID;
    lpszFilename:         LPSTR;
    lpszCommandLine:      LPSTR;
    lpszPath:             LPSTR;
    lpszCurrentDirectory: LPSTR;
    lpszLauncherName:     LPSTR;
  end;
  DIRECTXREGISTERAPP2A = TDIRECTXREGISTERAPP2A;

  PDIRECTXREGISTERAPPW = ^TDIRECTXREGISTERAPPW;
  TDIRECTXREGISTERAPPW = record
    dwSize:               DWORD;
    dwFlags:              DWORD;
    lpszApplicationName:  PWideChar;
    lpGUID:               PGUID;
    lpszFilename:         PWideChar;
    lpszCommandLine:      PWideChar;
    lpszPath:             PWideChar;
    lpszCurrentDirectory: PWideChar;
  end;
  DIRECTXREGISTERAPPW = TDIRECTXREGISTERAPPW;

  PDIRECTXREGISTERAPP2W = ^TDIRECTXREGISTERAPP2W;
  TDIRECTXREGISTERAPP2W = record
    dwSize:               DWORD;
    dwFlags:              DWORD;
    lpszApplicationName:  PWideChar;
    lpGUID:               PGUID;
    lpszFilename:         PWideChar;
    lpszCommandLine:      PWideChar;
    lpszPath:             PWideChar;
    lpszCurrentDirectory: PWideChar;
    lpszLauncherName:     PWideChar;
  end;
  DIRECTXREGISTERAPP2W = TDIRECTXREGISTERAPP2W;

  {$IFDEF UNICODE}
  TDIRECTXREGISTERAPP = TDIRECTXREGISTERAPPW;
  TDIRECTXREGISTERAPP2 = TDIRECTXREGISTERAPP2W;
  {$ELSE}
  TDIRECTXREGISTERAPP = TDIRECTXREGISTERAPPA;
  TDIRECTXREGISTERAPP2 = TDIRECTXREGISTERAPP2A;
  {$ENDIF}

//
// API
//

{$IFNDEF UNICODE}
function DirectXSetup(
  hWnd:         HWND;
  lpszRootPath: LPSTR;
  dwFlags:      DWORD
): Integer; stdcall; external 'dsetup.dll' name 'DirecXSetupA';
{$ELSE}
function DirectXSetup(
  hWnd:         HWND;
  lpszRootPath: PWideChar;
  dwFlags:      DWORD
): Integer; stdcall; external 'dsetup.dll' name 'DirecXSetupW';
{$ENDIF}

{$IFNDEF UNICODE}
function DirectXDeviceDriverSetup(
  hWnd:            HWND;
  lpszDriverClass: LPSTR;
  lpszDriverPath:  LPSTR;
  dwFlags:         DWORD
): Integer; stdcall; external 'dsetup.dll' name 'DirectXDeviceDriverSetupA';
{$ELSE}
function DirectXDeviceDriverSetup(
  hWnd:            HWND;
  lpszDriverClass: PWideChar;
  lpszDriverPath:  PWideChar;
  dwFlags:         DWORD
): Integer; stdcall; external 'dsetup.dll' name 'DirectXDeviceDriverSetupW';
{$ENDIF}

{$IFNDEF UNICODE}
function DirectXRegisterApplication(
  hWnd: HWND;
  lpDXRegApp: Pointer
): Integer; stdcall; external 'dsetup.dll' name 'DirectXRegisterApplicationA';
{$ELSE}
function DirectXRegisterApplication(
  hWnd: HWND;
  lpDXRegApp: Pointer
): Integer; stdcall; external 'dsetup.dll' name 'DirectXRegisterApplicationW';
{$ENDIF}

function DirectXUnRegisterApplication(
  hWnd:   HWND;
  lpGUID: PGUID
): Integer; stdcall; external 'dsetup.dll' name 'DirectXUnRegisterApplication';

type
  TDSETUP_CALLBACK = function(Reason: DWORD; MsgType: DWORD; szMessage: LPSTR;
                              szName: LPSTR; pInfo: Pointer): DWORD;
  DSETUP_CALLBACK = TDSETUP_CALLBACK;

function DirectXSetupSetCallback(Callback: DSETUP_CALLBACK): Integer; stdcall; external 'dsetup.dll' name 'DirectXSetupSetCallback';
function DirectXSetupGetVersion(lpdwVersion: PDWORD;lpdwMinorVersion: PDWORD): Integer; stdcall; external 'dsetup.dll' name 'DirectXSetupGetVersion';

implementation

end.
