//------------------------------------------------------------------------------
// :     basedd7.pas
//
// :       DirectDraw API
//
//
// ,     , 2002
//------------------------------------------------------------------------------
unit basedd7;

interface

uses
  windows,
  directdraw;



  
//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
const
  SCREEN_WIDTH  = 640; //  
  SCREEN_HEIGHT = 480; //  
  SPRITE_WIDTH  = 100; //  
  SPRITE_HEIGHT = 100; //  




//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
procedure ReleaseDirectDraw();
function LoadFiles( hWnd: HWND ): Boolean;
procedure OnDraw( nX, nY: Integer );

implementation

uses
  ddutils;




//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
procedure SAFE_DELETE( p: TInterfacedObject ); begin if p <> nil then p := nil end;

var
  pDD7:            IDirectDraw7        = nil;  //   DirectDraw
  pPrimarySurface: IDirectDrawSurface7 = nil;  //  
  pBackBuffer:     IDirectDrawSurface7 = nil;  //  

  pWallpaper:      IDirectDrawSurface7 = nil;  //  
  pSprite:         IDirectDrawSurface7 = nil;  // 




//------------------------------------------------------------------------------
// :      InitDirectDraw()
// :    DirectDraw  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  Result := FALSE;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, pDD7, IID_IDirectDraw7, nil ) ) then
     Exit;

  //   
  if FAILED( pDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                             DDSCL_EXCLUSIVE or
                                             DDSCL_FULLSCREEN ) ) then
     Exit;

  //      
  if FAILED( pDD7.SetDisplayMode( dwWidth, dwHeight, dwBpp, 0, 0 ) ) then
     Exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( pDD7.CreateSurface( ddsd2, pPrimarySurface, nil ) ) then
     Exit;

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, SizeOf( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  //      
  if FAILED( pPrimarySurface.GetAttachedSurface( ddsc2, pBackBuffer ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectDraw()
// :     DirectDraw
//------------------------------------------------------------------------------
procedure ReleaseDirectDraw();
begin
  SAFE_DELETE( @pSprite );
  SAFE_DELETE( @pWallpaper );
  SAFE_DELETE( @pBackBuffer );
  SAFE_DELETE( @pPrimarySurface );

  if pDD7 <> nil then
  begin
    pDD7.RestoreDisplayMode();
    pDD7 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      LoadFiles()
// :    bmp-   DirectDraw
//------------------------------------------------------------------------------
function LoadFiles( hWnd: HWND ): Boolean;
var
  hBmp: HBITMAP;
begin
   Result := FALSE;

   hBmp := LoadImage( 0, 'wallpaper.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pWallpaper, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   hBmp := LoadImage( 0, 'sprite.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pSprite, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      OnDraw()
// :       
//------------------------------------------------------------------------------
procedure OnDraw( nX, nY: Integer );
var
  rRect: TRect;
begin
  pBackBuffer.BltFast( 0, 0, pWallpaper, nil, DDBLTFAST_WAIT );

  //----------------------------------------------------------------------------
  
  //     ,    .  
  //     IDirectDrawClipper   
  //      .

  // ,        
  SetRect( rRect, 0, 0, SPRITE_WIDTH, SPRITE_HEIGHT );

  //   ,      ,  
  //      
  if nX < 0 then rRect.Left := - nX ;
  if nY < 0 then rRect.Top := - nY;
  if nX + SPRITE_WIDTH > SCREEN_WIDTH then rRect.Right := SCREEN_WIDTH - nX;
  if nY + SPRITE_HEIGHT > SCREEN_HEIGHT then rRect.Bottom := SCREEN_HEIGHT - nY;

  //----------------------------------------------------------------------------

  //    ,     BltFast(),
  //    .      ,   
  // ?
  if ( nX < 0 ) and ( nX + SPRITE_WIDTH < 0 ) or
     ( nY < 0 ) and ( nY + SPRITE_HEIGHT < 0 ) or
     ( nX > SCREEN_WIDTH ) or
     ( nY > SCREEN_HEIGHT )
  then MessageBeep( $FFFFFFFF ) //      
  else pBackBuffer.BltFast( nX + rRect.Left,
                            nY + rRect.Top,
                            pSprite,
                            @rRect,
                            DDBLTFAST_WAIT );

  //         
  pPrimarySurface.Flip( nil, DDFLIP_WAIT );
end;

end.
