//------------------------------------------------------------------------------
// :     basedd7.pas
//
// :       DirectDraw API
//
//
// ,     , 2002
//------------------------------------------------------------------------------
unit basedd7;

interface

uses
  windows,
  sysutils,
  directdraw;




//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
procedure ReleaseDirectDraw();
procedure TimerProc( uTimerID, uMessage: UINT; dwUser, dw1, dw2: DWORD ); stdcall;
function LoadFiles( hWnd: HWND ): Boolean;
procedure OnDraw( nX, nY: Integer );

implementation

uses
  ddutils;




//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
procedure SAFE_DELETE( p: TInterfacedObject ); begin if p <> nil then p := nil end;

var
  pDD7:            IDirectDraw7        = nil;  //   DirectDraw
  pPrimarySurface: IDirectDrawSurface7 = nil;  //  
  pBackBuffer:     IDirectDrawSurface7 = nil;  //  

  pWallpaper: IDirectDrawSurface7 = nil;  //  
  pSprite:    IDirectDrawSurface7 = nil;  // 
  pFPS:       IDirectDrawSurface7 = nil;

  dwFPSValue: DWORD;




//------------------------------------------------------------------------------
// :      InitDirectDraw()
// :    DirectDraw  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  Result := FALSE;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, pDD7, IID_IDirectDraw7, nil ) ) then
     Exit;

  //   
  if FAILED( pDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                             DDSCL_EXCLUSIVE or
                                             DDSCL_FULLSCREEN ) ) then
     Exit;

  //      
  if FAILED( pDD7.SetDisplayMode( dwWidth, dwHeight, dwBpp, 0, 0 ) ) then
     Exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( pDD7.CreateSurface( ddsd2, pPrimarySurface, nil ) ) then
     Exit;

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, SizeOf( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  //      
  if FAILED( pPrimarySurface.GetAttachedSurface( ddsc2, pBackBuffer ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectDraw()
// :     DirectDraw
//------------------------------------------------------------------------------
procedure ReleaseDirectDraw();
begin
  SAFE_DELETE( @pFPS );
  SAFE_DELETE( @pSprite );
  SAFE_DELETE( @pBackBuffer );
  SAFE_DELETE( @pPrimarySurface );

  if pDD7 <> nil then
  begin
    pDD7.RestoreDisplayMode();
    pDD7 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      LoadFiles()
// :    bmp-   DirectDraw
//------------------------------------------------------------------------------
function LoadFiles( hWnd: HWND ): Boolean;
var
  hBmp: HBITMAP;
  ddck: TDDCOLORKEY;
begin
   Result := FALSE;

   hBmp := LoadImage( 0, 'wallpaper.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pWallpaper, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   hBmp := LoadImage( 0, 'sprite.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pSprite, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   //     fps
   if not CreateSurface( pDD7, pFPS, 100, 100 ) then
      Exit;

   // 
   ddck.dwColorSpaceLowValue  := 0;
   ddck.dwColorSpaceHighValue := ddck.dwColorSpaceLowValue;
   pFPS.SetColorKey( DDCKEY_SRCBLT, @ddck );

   Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      OutText()
// :   fps   DirectDraw
//------------------------------------------------------------------------------
procedure OutText( pSurf: IDirectDrawSurface7; nPosX, nPosY: Integer; strText: string );
var
  DC: HDC;
begin
  //    GDI- DC  
  if SUCCEEDED( pSurf.GetDC( DC ) ) then
  begin
    //     ( )
    PatBlt( DC, 0, 0, 100, 100, BLACKNESS );

    //    
    SetTextColor( DC, RGB( 255, 0, 0 ) );

    //     
    SetBkMode( DC, TRANSPARENT );

    //      DirectDraw
    TextOut( DC,                  //   ( DirectDraw)
             nPosX,               //   
             nPosY,               //   
             PChar( strText ),    //  
             Length( strText ) ); //  

    //  
    pSurf.ReleaseDC( DC );
  end;
end;




//------------------------------------------------------------------------------
// :      MMTimerProc()
// : Callback- ,  1   
//------------------------------------------------------------------------------
procedure TimerProc( uTimerID, uMessage: UINT; dwUser, dw1, dw2: DWORD ); stdcall;
begin
  //    pFPS  fps
  OutText( pFPS, 0, 0, Format( 'fps %d', [dwFPSValue] ) );
  dwFPSValue := 0;
end;




//------------------------------------------------------------------------------
// :      OnDraw()
// :     
//------------------------------------------------------------------------------
procedure OnDraw( nX, nY: Integer );
begin
  //     
  pBackBuffer.BltFast( 0, 0, pWallpaper, nil, DDBLTFAST_WAIT );
  pBackBuffer.BltFast( nX, nY, pSprite, nil, DDBLTFAST_WAIT );

  //   fps
  pBackBuffer.BltFast( 20, 20, pFPS, nil, DDBLTFAST_SRCCOLORKEY or DDBLTFAST_WAIT );

  //         
  pPrimarySurface.Flip( nil, DDFLIP_WAIT );

  Inc( dwFPSValue ); //   -   fsp
end;

end.
