//------------------------------------------------------------------------------
// :     ddutils.pas
//
// :      DirectDraw
//
//
// ,     , 2002
//------------------------------------------------------------------------------
unit ddutils;

interface

uses
  windows,
  directdraw;




//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
function CreateSurface( var pDD7: IDirectDraw7; var pSurface: IDirectDrawSurface7;
                        nWidth, nHeight: Integer ): Boolean;

function CopyBitmap( var pDD7: IDirectDraw7; var pSurf: IDirectDrawSurface7;
                     const hBmp: HBITMAP ): Boolean;

implementation




//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
procedure SAFE_DELETE( p: TInterfacedObject ); begin if p <> nil then p := nil end;




//------------------------------------------------------------------------------
// :      CreateSurface()
// :   DirectDraw  
//------------------------------------------------------------------------------
function CreateSurface( var pDD7: IDirectDraw7; var pSurface: IDirectDrawSurface7;
                        nWidth, nHeight: Integer ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
begin
  Result := FALSE;

  //   ,  
  SAFE_DELETE( @pSurface );

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  with ddsd2 do
  begin
    dwSize         := SizeOf( TDDSURFACEDESC2 );
    dwFlags        := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT;
    ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN;
    dwWidth        := nWidth;
    dwHeight       := nHeight;
  end;

  //  
  if FAILED( pDD7.CreateSurface( ddsd2, pSurface, nil ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      CopyBitmap()
// :     DirectDraw
//------------------------------------------------------------------------------
function CopyBitmap( var pDD7: IDirectDraw7; var pSurf: IDirectDrawSurface7;
                     const hBmp: HBITMAP ): Boolean;
var
  destDC,          // -
  srcDC:  HDC;     // -
  bm:     BITMAP;  // BITMAP  windows.pas
begin
  Result := FALSE;

  if pDD7 = nil then
     Exit;

  //   ,    
  srcDC := CreateCompatibleDC( 0 );
  if srcDC = 0 then
     Exit;

  //    
  SelectObject( srcDC, hBmp );

  //   ,    BITMAP
  if GetObject( hBmp, SizeOf( bm ), @bm ) = 0 then
     Exit;

  //     
  if not CreateSurface( pDD7, pSurf, bm.bmWidth, bm.bmHeight ) then
     Exit;

  //  GDI-   
  if SUCCEEDED( pSurf.GetDC( destDC ) ) then
  begin
    // BitBlt-   (    DirectDraw)
    BitBlt( destDC, 0, 0, bm.bmWidth, bm.bmHeight, srcDC, 0, 0, SRCCOPY );
    pSurf.ReleaseDC( destDC );
  end
  else
  begin
    DeleteDC( srcDC );
    Exit;
  end;

  //      GDI
  DeleteDC( srcDC );

  Result := TRUE;
end;

end.
