//------------------------------------------------------------------------------
// :     basedd7.pas
//
// :       DirectDraw API
//
//
// ,     , 2002
//------------------------------------------------------------------------------
unit basedd7;

interface

uses
  windows,
  directdraw;




//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
procedure ReleaseDirectDraw();
function LoadFiles( hWnd: HWND ): Boolean;
procedure OnDraw( nX, nY: Integer );

implementation

uses
  ddutils;




//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
procedure SAFE_DELETE( p: TInterfacedObject ); begin if p <> nil then p := nil end;

var
  pDD7:            IDirectDraw7        = nil;  //   DirectDraw
  pPrimarySurface: IDirectDrawSurface7 = nil;  //  
  pBackBuffer:     IDirectDrawSurface7 = nil;  //  

  pWallpaper: IDirectDrawSurface7 = nil;  //  
  pSprite:    IDirectDrawSurface7 = nil;  // 




//------------------------------------------------------------------------------
// :      InitDirectDraw()
// :    DirectDraw  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  Result := FALSE;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, pDD7, IID_IDirectDraw7, nil ) ) then
     Exit;

  //   
  if FAILED( pDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                             DDSCL_EXCLUSIVE or
                                             DDSCL_FULLSCREEN ) ) then
     Exit;

  //      
  if FAILED( pDD7.SetDisplayMode( dwWidth, dwHeight, dwBpp, 0, 0 ) ) then
     Exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( pDD7.CreateSurface( ddsd2, pPrimarySurface, nil ) ) then
     Exit;

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, SizeOf( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  //      
  if FAILED( pPrimarySurface.GetAttachedSurface( ddsc2, pBackBuffer ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectDraw()
// :     DirectDraw
//------------------------------------------------------------------------------
procedure ReleaseDirectDraw();
begin
  SAFE_DELETE( @pSprite );
  SAFE_DELETE( @pBackBuffer );
  SAFE_DELETE( @pPrimarySurface );

  if pDD7 <> nil then
  begin
    pDD7.RestoreDisplayMode();
    pDD7 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      LoadFiles()
// :    bmp-   DirectDraw
//------------------------------------------------------------------------------
function LoadFiles( hWnd: HWND ): Boolean;
var
  hBmp: HBITMAP;
begin
   Result := FALSE;

   hBmp := LoadImage( 0, 'wallpaper.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pWallpaper, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   hBmp := LoadImage( 0, 'sprite.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pSprite, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      OutText()
// :    DDraw      GDI
//------------------------------------------------------------------------------
procedure OutText( pSurf: IDirectDrawSurface7; nPosX, nPosY: Integer; strText: string );
var
  DC:         HDC;
  hFontStyle: HFONT;
begin
  //    GDI- DC  
  if SUCCEEDED( pSurf.GetDC( DC ) ) then
  begin
    //    
    SetTextColor( DC, RGB( Random( 255 ), Random( 255 ), Random( 255 ) ) );

    //     
    SetBkMode( DC, TRANSPARENT );

    //   
    hFontStyle := CreateFont( 40,
                              0,
                              0,
                              0,
                              FW_EXTRABOLD,
                              0,
                              0,
                              0,
                              RUSSIAN_CHARSET or ANSI_CHARSET,
                              OUT_DEFAULT_PRECIS,
                              CLIP_DEFAULT_PRECIS,
                              DEFAULT_QUALITY{ANTIALIASED_QUALITY},
                              VARIABLE_PITCH,
                              'Times New Roman' );

    SelectObject( DC, hFontStyle );

    //      DirectDraw
    TextOut( DC,                  //   ( DirectDraw)
             nPosX,               //   
             nPosY,               //   
             PChar( strText ),    //  
             Length( strText ) ); //  

    //  
    pSurf.ReleaseDC( DC );

    //   -
    DeleteObject( hFontStyle );
  end;
end;




//------------------------------------------------------------------------------
// :      OnDraw()
// :     
//------------------------------------------------------------------------------
procedure OnDraw( nX, nY: Integer );
begin
  //     
  pBackBuffer.BltFast( 0, 0, pWallpaper, nil, DDBLTFAST_WAIT );
  pBackBuffer.BltFast( nX, nY, pSprite, nil, DDBLTFAST_WAIT );

  //  
  OutText( pBackBuffer, 25, 100, 'GDI  DirectDraw -  ' );

  //         
  pPrimarySurface.Flip( nil, DDFLIP_WAIT );
end;

end.
