//------------------------------------------------------------------------------
// :     basedd7.pas
//
// :       DirectDraw API
//
//
// ,     , 2002
//------------------------------------------------------------------------------
unit basedd7;

interface

uses
  windows,
  directdraw;




//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
procedure ReleaseDirectDraw();
function LoadFiles( hWnd: HWND ): Boolean;
procedure OnDraw( nX, nY: Integer );

implementation

uses
  ddutils;




//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
procedure SAFE_DELETE( p: TInterfacedObject  ); begin if p <> nil then p := nil end;
function RGB565( r, g, b: Byte ): COLORREF; begin Result := ( ( r shl 11 ) or ( g shl 5 ) or b ); end;
function RGB555( r, g, b: Byte ): COLORREF; begin Result := ( ( r shl 10 ) or ( g shl 5 ) or b ); end;

const
  GM_16_565 = 0;
  GM_16_555 = 1;

var
  pDD7:            IDirectDraw7        = nil;  //   DirectDraw
  pPrimarySurface: IDirectDrawSurface7 = nil;  //  
  pBackBuffer:     IDirectDrawSurface7 = nil;  //  

  pWallpaper: IDirectDrawSurface7 = nil;  //  
  pSprite:    IDirectDrawSurface7 = nil;  // 




//------------------------------------------------------------------------------
// :      InitDirectDraw()
// :    DirectDraw  
//------------------------------------------------------------------------------
function InitDirectDraw( hWnd: HWND; dwWidth, dwHeight, dwBPP: DWORD ): Boolean;
var
  ddsd2: TDDSURFACEDESC2;
  ddsc2: TDDSCAPS2;
begin
  Result := FALSE;

  //    DirectDraw
  if FAILED( DirectDrawCreateEx( nil, pDD7, IID_IDirectDraw7, nil ) ) then
     Exit;

  //   
  if FAILED( pDD7.SetCooperativeLevel( hWnd, DDSCL_ALLOWREBOOT or
                                             DDSCL_EXCLUSIVE or
                                             DDSCL_FULLSCREEN ) ) then
     Exit;

  //      
  if FAILED( pDD7.SetDisplayMode( dwWidth, dwHeight, dwBpp, 0, 0 ) ) then
     Exit;

  //   TDDSURFACEDESC2
  ZeroMemory( @ddsd2, SizeOf( TDDSURFACEDESC2 ) );
  ddsd2.dwSize := SizeOf( TDDSURFACEDESC2 );

  ddsd2.dwFlags := DDSD_CAPS or DDSD_BACKBUFFERCOUNT;
  ddsd2.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE or
                          DDSCAPS_FLIP or
                          DDSCAPS_COMPLEX;
  ddsd2.dwBackBufferCount := 1;

  //    (    )
  if FAILED( pDD7.CreateSurface( ddsd2, pPrimarySurface, nil ) ) then
     Exit;

  //   TDDSCAPS2
  ZeroMemory( @ddsc2, SizeOf( TDDSCAPS2 ) );
  ddsc2.dwCaps := DDSCAPS_BACKBUFFER;

  //      
  if FAILED( pPrimarySurface.GetAttachedSurface( ddsc2, pBackBuffer ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      ReleaseDirectDraw()
// :     DirectDraw
//------------------------------------------------------------------------------
procedure ReleaseDirectDraw();
begin
  SAFE_DELETE( @pSprite );
  SAFE_DELETE( @pWallpaper );
  SAFE_DELETE( @pBackBuffer );
  SAFE_DELETE( @pPrimarySurface );

  if pDD7 <> nil then
  begin
    pDD7.RestoreDisplayMode();
    pDD7 := nil;
  end;
end;




//------------------------------------------------------------------------------
// :      DDGetGraphicsMode()
// :   
//------------------------------------------------------------------------------
function DDGetGraphicsMode( var bGraphicsMode: Byte ): Boolean;
var
  pTest: IDirectDrawSurface7;
  ddpf:  TDDPIXELFORMAT;
begin
  Result := FALSE;

  pTest := nil;
  if not CreateSurface( pDD7, pTest, 1, 1 ) then
     Exit;

  //   TDDPIXELFORMAT
  ZeroMemory( @ddpf, SizeOf( TDDPIXELFORMAT ) );
  ddpf.dwSize := SizeOf( TDDPIXELFORMAT );

  //  
  if FAILED( pTest.GetPixelFormat( ddpf ) ) then
  begin
    pTest := nil;
    Exit;
  end;

  //      16- 
  if ddpf.dwRGBBitCount <> 16 then
  begin
    pTest := nil;
    Exit;
  end;

  if ( ddpf.dwRBitMask = $0000F800 ) and
     ( ddpf.dwGBitMask = $000007E0 ) and
     ( ddpf.dwBBitMask = $0000001F ) then
     bGraphicsMode := GM_16_565
  else
  if ( ddpf.dwRBitMask = $00007C00 ) and
     ( ddpf.dwGBitMask = $000003E0 ) and
     ( ddpf.dwBBitMask = $0000001F ) then
     bGraphicsMode := GM_16_555
  else Exit;

  pTest := nil;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      LoadFiles()
// :    bmp-   DirectDraw
//------------------------------------------------------------------------------
function LoadFiles( hWnd: HWND ): Boolean;
var
  hBmp: HBITMAP;
  ddck: TDDCOLORKEY;

  bGraphicsMode: Byte;
begin
   Result := FALSE;

   hBmp := LoadImage( 0, 'wallpaper.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pWallpaper, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   hBmp := LoadImage( 0, 'sprite.bmp', IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE or LR_CREATEDIBSECTION );
   if not CopyBitmap( pDD7, pSprite, hBmp ) then
      Exit;
   DeleteObject( hBmp );

   //   
   if not DDGetGraphicsMode( bGraphicsMode ) then
      Exit;

   //    - 
   case  bGraphicsMode of
     GM_16_565: ddck.dwColorSpaceLowValue := RGB565( 0, 63, 0 );
     GM_16_555: ddck.dwColorSpaceLowValue := RGB555( 0, 31, 0 );
   end;
   ddck.dwColorSpaceHighValue := ddck.dwColorSpaceLowValue;

   //   
   pSprite.SetColorKey( DDCKEY_SRCBLT, @ddck );

   Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      OnDraw()
// :     
//------------------------------------------------------------------------------
procedure OnDraw( nX, nY: Integer );
begin
  //     
  pBackBuffer.BltFast( 0, 0, pWallpaper, nil, DDBLTFAST_WAIT );

  //  DDBLTFAST_SRCCOLORKEY     
  //    -
  pBackBuffer.BltFast( nX, nY, pSprite, nil, DDBLTFAST_SRCCOLORKEY or DDBLTFAST_WAIT );

  //         
  pPrimarySurface.Flip( nil, DDFLIP_WAIT );
end;

end.
