//------------------------------------------------------------------------------
// :     lowfunc.pas
//
// :        wav-.
//             Max Morozov  iXania    
//               ++  Object Pascal
//
// ,     , 2002
//------------------------------------------------------------------------------
unit lowfunc;

interface

uses
  windows,
  mmsystem;




//------------------------------------------------------------------------------
// ,    
//------------------------------------------------------------------------------
type
  PPWAVEFORMATEX = ^PWAVEFORMATEX; // WAVEFORMATEX**


function WaveOpenFile(
  strFileName: PChar;          //    (c  .wav)
  phmmioIn:    PHMMIO;
  ppwfxInfo:   PPWAVEFORMATEX;
  pckInRIFF:   PMMCKINFO
): HRESULT;

function WaveStartDataRead(
  phmmioIn:  PHMMIO;
  pckIn:     PMMCKINFO;
  pckInRIFF: PMMCKINFO
): HRESULT;

function WaveReadFile(
  hmmioIn:      HMMIO;
  cbRead:       UINT;
  pbDest:       PByte;
  pckIn:        PMMCKINFO;
  cbActualRead: PUINT
): HRESULT;

implementation




//------------------------------------------------------------------------------
// :      ReadMMIO()
// :     -
//------------------------------------------------------------------------------
function ReadMMIO( hmmioIn: HMMIO; pckInRIFF: PMMCKINFO;
                   ppwfxInfo: PPWAVEFORMATEX ): HRESULT;
var
  ckIn:         MMCKINFO;
  pcmWavFormat: PCMWAVEFORMAT;
  cbExtraBytes: WORD;
begin
  Result := E_FAIL;

  ppwfxInfo^ := nil;

  if mmioDescend( hmmioIn, pckInRIFF, nil, 0 ) <> 0 then
     Exit;

  if ( pckInRIFF.ckid <> FOURCC_RIFF ) or
     ( pckInRIFF.fccType <> mmioStringToFOURCC( 'wave', MMIO_TOUPPER ) ) then
     Exit;                  // mmioFOURCC()  mmsystem.pas  

  ckIn.ckid := mmioStringToFOURCC( 'fmt ', 0 );
  if mmioDescend( hmmioIn, @ckIn, pckInRIFF, MMIO_FINDCHUNK ) <> 0 then
     Exit;

  if ckIn.cksize < DWORD(SizeOf( PCMWAVEFORMAT )) then
     Exit;

  if mmioRead( hmmioIn, PChar( @pcmWavFormat ),
              SizeOf( PCMWAVEFORMAT ) ) <> SizeOf( PCMWAVEFORMAT ) then
     Exit;

  if pcmWavFormat.wf.wFormatTag = WAVE_FORMAT_PCM then
  begin
    GetMem( ppwfxInfo^, SizeOf( TWAVEFORMATEX ) );
    CopyMemory( ppwfxInfo^, @pcmWavFormat, SizeOf( PCMWAVEFORMAT ) );
    ppwfxInfo^.cbSize := 0;
  end
  else  //  WAVE_PCM_FORMAT
  begin
    cbExtraBytes := 0;

    if mmioRead( hmmioIn, PChar( @cbExtraBytes ), SizeOf( WORD ) ) <> SizeOf( WORD ) then
       Exit;

    GetMem( ppwfxInfo^, SizeOf( TWAVEFORMATEX )+ cbExtraBytes );

    if ppwfxInfo^ = nil then
       Exit;

    CopyMemory( ppwfxInfo^, @pcmWavFormat, SizeOf( PCMWAVEFORMAT ) );
    ppwfxInfo^.cbSize := cbExtraBytes;

    if mmioRead( hmmioIn, PChar(PByte((ppwfxInfo^.cbSize) + SizeOf( WORD ))),
                 cbExtraBytes ) <> cbExtraBytes then
    begin
      Dispose( ppwfxInfo^ );
      ppwfxInfo^ := nil;
      Result := E_FAIL;
      Exit;
    end;
  end;

  if mmioAscend( hmmioIn, @ckIn, 0 ) <> 0 then
  begin
    FreeMem( ppwfxInfo^, SizeOf( TWAVEFORMATEX )+ cbExtraBytes ); // GetMem()
    ppwfxInfo^ := nil;
    Exit;
  end;

  Result := S_OK;
end;




//------------------------------------------------------------------------------
// :      WaveOpenFile()
// :       
//------------------------------------------------------------------------------
function WaveOpenFile( strFileName: PChar; phmmioIn: PHMMIO;
                       ppwfxInfo: PPWAVEFORMATEX; pckInRIFF: PMMCKINFO ): HRESULT;
var
  hmmioIn: HMMIO;
begin
  Result := E_FAIL;

  hmmioIn := 0;

  hmmioIn := mmioOpen( strFileName, nil, MMIO_ALLOCBUF or MMIO_READ );

  if hmmioIn = 0 then
     Exit;

  if FAILED( ReadMMIO( hmmioIn, pckInRIFF, ppwfxInfo ) ) then
  begin
    mmioClose( hmmioIn, 0 );
    Exit;
  end;

  phmmioIn^ := hmmioIn;

  Result := S_OK;
end;




//------------------------------------------------------------------------------
// :      WaveStartDataRead()
// :   'data'      
//------------------------------------------------------------------------------
function WaveStartDataRead( phmmioIn: PHMMIO; pckIn: PMMCKINFO;
                            pckInRIFF: PMMCKINFO ): HRESULT;
begin
  Result := E_FAIL;

  if mmioSeek( phmmioIn^, pckInRIFF.dwDataOffset+SizeOf( FOURCC ),
               SEEK_SET ) = -1 then
     Exit;

  pckIn.ckid := mmioStringToFOURCC( 'data', 0 );
  if mmioDescend ( phmmioIn^, pckIn, pckInRIFF, MMIO_FINDCHUNK ) <> 0 then
     Exit;

  Result := S_OK;
end;




//------------------------------------------------------------------------------
// :      WaveReadFile()
// :     wav-
//------------------------------------------------------------------------------
function WaveReadFile( hmmioIn: HMMIO; cbRead: UINT; pbDest: PByte;
                       pckIn: PMMCKINFO; cbActualRead: PUINT ): HRESULT;
var
  mmioinfoIn: MMIOINFO;
  cbDataIn:   UINT;
  cT:         DWORD;
begin
  Result := E_FAIL;

  cbActualRead^ := 0;

  if mmioGetInfo( hmmioIn, @mmioinfoIn, 0 ) <> 0 then
     Exit;

  cbDataIn := cbRead;

  if cbDataIn > pckIn.cksize then
     cbDataIn := pckIn.cksize;

  pckIn.cksize := pckIn.cksize - cbDataIn;

  cT := 0;

  while cT < cbDataIn do
  begin
    //    I/O  
    if mmioinfoIn.pchNext = mmioinfoIn.pchEndRead then
    begin
      if mmioAdvance( hmmioIn, @mmioinfoIn, MMIO_READ ) <> 0 then
         Exit;

      if mmioinfoIn.pchNext = mmioinfoIn.pchEndRead then
         Exit;
    end;

    PByte(Pointer(Integer(pbDest) + cT))^ := PByte( mmioinfoIn.pchNext )^;

    Inc( mmioinfoIn.pchNext );
    Inc( cT );
  end;

  if mmioSetInfo( hmmioIn, @mmioinfoIn, 0 ) <> 0 then
  begin
    Result := E_FAIL; Exit;
  end;

  cbActualRead^ := cbDataIn;

  Result := S_OK;
end;

end.
