//------------------------------------------------------------------------------
// :     wave.pas
//
// :      DirectSound
//
//
// ,     , 2002
//------------------------------------------------------------------------------
unit wave;

interface

uses
  windows,
  mmsystem,
  directsound;

type
  TWaveSoundRead = class
  public
    m_pwfx:     PWAVEFORMATEX;   //    WAVEFRMATEX
    m_hmmioIn:  HMMIO;           //   *.wav
    m_ckIn:     MMCKINFO;        //  RIFF
    m_ckInRIFF: MMCKINFO;        //    *.wav-
  public
    constructor Create();
    destructor  Destroy();
    function Open( strFileName: string ): Boolean;
    function Reset(): Boolean;
    function Read( nSizeToRead: UINT; pbData: PByte; pnSizeRead: PUINT ): Boolean;
    function Close(): Boolean;
  end;

  //

  TWave = class
  private
    wsr:          TWaveSoundRead;
    pSoundBuffer: IDirectSoundBuffer;
    bbytes:       Integer;
    function FillBuffer(): Boolean;
  public
    constructor Create();
    destructor Destroy();
    function OpenWaveFile( strFileName: string ): Boolean;
    function Play( bLoop: Boolean ): Boolean;
    procedure Stop();
    procedure GetStatus( var dwStatus: DWORD );
    procedure SetVolume( nVolume: Integer );
    procedure SetPan( nPan: Integer );
    procedure SetCurrentPosition( dwPosition: DWORD );
  end;

implementation

uses
  main,
  lowfunc;




(*
 *   TWaveSoundRead
 *)




//------------------------------------------------------------------------------
// :      TWaveSoundRead.Create()
// :  
//------------------------------------------------------------------------------
constructor TWaveSoundRead.Create();
begin
  inherited Create();

  m_pwfx := nil;
end;




//------------------------------------------------------------------------------
// :      TWaveSoundRead.Destroy()
// :  
//------------------------------------------------------------------------------
destructor TWaveSoundRead.Destroy();
begin
  Close();
  m_pwfx := nil;

  inherited Destroy();
end;




//------------------------------------------------------------------------------
// :      TWaveSoundRead.Open()
// :    
//------------------------------------------------------------------------------
function TWaveSoundRead.Open( strFileName: string ): Boolean;
begin
  Result := FALSE;

  m_pwfx := nil;

  if FAILED( WaveOpenFile( PChar( strFileName ), @m_hmmioIn, @m_pwfx,
                           @m_ckInRIFF) ) then
     Exit;

  if not Reset() then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TWaveSoundRead.Reset()
// :       
//------------------------------------------------------------------------------
function TWaveSoundRead.Reset(): Boolean;
begin
  Result := not FAILED( WaveStartDataRead( @m_hmmioIn, @m_ckIn, @m_ckInRIFF ) );
end;




//------------------------------------------------------------------------------
// :      TWaveSoundRead.Read()
// :    
//------------------------------------------------------------------------------
function TWaveSoundRead.Read( nSizeToRead: UINT; pbData: PByte;
                              pnSizeRead: PUINT ): Boolean;
begin
  Result := not FAILED( WaveReadFile( m_hmmioIn, nSizeToRead, pbData, @m_ckIn,
                                      pnSizeRead ) );
end;




//------------------------------------------------------------------------------
// :      TWaveSoundRead.Close()
// :  
//------------------------------------------------------------------------------
function TWaveSoundRead.Close(): Boolean;
begin
  mmioClose(m_hmmioIn, 0);
  Result := TRUE;
end;




(*
 *   TWave
 *)




//------------------------------------------------------------------------------
// :      TWave.Create()
// :  
//------------------------------------------------------------------------------
constructor TWave.Create();
begin
  inherited Create();

  pSoundBuffer := nil;
  wsr := nil;
  bbytes := 0;
end;




//------------------------------------------------------------------------------
// :      TWave.Destroy()
// :  
//------------------------------------------------------------------------------
destructor TWave.Destroy();
begin
  if pSoundBuffer <> nil then
     pSoundBuffer := nil;

  if wsr <> nil then
     wsr.Destroy();

  inherited Destroy();
end;




//------------------------------------------------------------------------------
// :      TWave.FillBuffer()
// :    DirectSound   wav-
//------------------------------------------------------------------------------
function TWave.FillBuffer(): Boolean;
var
  pbWavData: PByte;
  cbWavSize: UINT;
  pbData:    Pointer;
  pbData2:   Pointer;
  dwLength:  DWORD;
  dwLength2: DWORD;
  nWaveFileSize: Integer;
begin
  Result := FALSE;

  pbData  := nil;
  pbData2 := nil;

  //    
  nWaveFileSize := wsr.m_ckIn.cksize;

  GetMem( pbWavData, nWaveFileSize );

  // ,    
  if pbWavData = nil then
     Exit;

  //      
  if not wsr.Read( nWaveFileSize, pbWavData, @cbWavSize ) then
     Exit;

  //     
  wsr.Reset();

  //   (   )
  if FAILED( pSoundBuffer.Lock( 0, bbytes, pbData, dwLength, pbData2, dwLength2, 0 ) ) then
     Exit;

  //        DirectSound
  CopyMemory( pbData, pbWavData, bbytes );

  //    
  if FAILED( pSoundBuffer.Unlock( pbData, bbytes, nil, 0 ) ) then
     Exit;

  pbData := nil;

  //       ,  
  FreeMem( pbWavData, nWaveFileSize );
  pbWavData := nil;                    

  Result := TRUE;
end;



//------------------------------------------------------------------------------
// :      TWave.OpenWaveFile()
// :          
//------------------------------------------------------------------------------
function TWave.OpenWaveFile( strFileName: string ): Boolean;
var
  dsbd8: TDSBUFFERDESC;
begin
  Result := FALSE;

  if wsr <> nil then
     wsr := nil;

  wsr := TWaveSoundRead.Create();
  if not wsr.Open( strFileName ) then
     Exit;

  //   TDSBUFFERDESC_DX8
  ZeroMemory( @dsbd8, SizeOf( TDSBUFFERDESC ) );
  dsbd8.dwSize  := SizeOf( TDSBUFFERDESC );
  dsbd8.dwFlags := DSBCAPS_STATIC or DSBCAPS_CTRLPAN or DSBCAPS_CTRLVOLUME or
                   DSBCAPS_GLOBALFOCUS;
  dsbd8.dwBufferBytes := wsr.m_ckIn.cksize;
  dsbd8.lpwfxFormat   := wsr.m_pwfx;

  //     
  if FAILED( pDS8.CreateSoundBuffer( dsbd8, pSoundBuffer, nil ) ) then
     Exit;

  //       DirectSound
  bbytes := dsbd8.dwBufferBytes;

  if not FillBuffer() then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TWave.Play()
// :     
//------------------------------------------------------------------------------
function TWave.Play( bLoop: Boolean ): Boolean;
begin
  Result := FALSE;

  if pSoundBuffer = nil then
     Exit;

  if bLoop then
  if FAILED( pSoundBuffer.Play( 0, 0, DSBPLAY_LOOPING ) ) then
     Exit;

  if not bLoop then
  if FAILED( pSoundBuffer.Play( 0, 0, 0 ) ) then
     Exit;

  Result := TRUE;
end;




//------------------------------------------------------------------------------
// :      TWave.Stop()
// :   
//------------------------------------------------------------------------------
procedure TWave.Stop();
begin
  if pSoundBuffer <> nil then pSoundBuffer.Stop();
end;




//------------------------------------------------------------------------------
// :      TWave.GetStatus()
// :    
//------------------------------------------------------------------------------
procedure TWave.GetStatus( var dwStatus: DWORD );
begin
  pSoundBuffer.GetStatus( dwStatus );
end;




//------------------------------------------------------------------------------
// :      TWave.SetVolume()
// :    
//------------------------------------------------------------------------------
procedure TWave.SetVolume( nVolume: Integer );
begin
  //        
  if nVolume < DSBVOLUME_MIN then nVolume := DSBVOLUME_MIN;
  if nVolume > DSBVOLUME_MAX then nVolume := DSBVOLUME_MAX;

  if pSoundBuffer <> nil then pSoundBuffer.SetVolume( nVolume );
end;




//------------------------------------------------------------------------------
// :      TWave.SetPan()
// :        
//------------------------------------------------------------------------------
procedure TWave.SetPan( nPan: Integer );
begin
  //        
  if nPan < DSBPAN_LEFT then nPan := DSBPAN_LEFT;
  if nPan > DSBPAN_RIGHT then nPan := DSBPAN_RIGHT;

  if pSoundBuffer <> nil then pSoundBuffer.SetPan( nPan );
end;




//------------------------------------------------------------------------------
// :      TWave.SetCurrentPosition()
// :      . 0 -  
//------------------------------------------------------------------------------
procedure TWave.SetCurrentPosition( dwPosition: DWORD );
begin
  pSoundBuffer.SetCurrentPosition( dwPosition );
end;

end.
